/* =====================================
=  Program Name: abundancebin
=  File Name: main.cpp
=  Version: 1.0
=  Developer: Yu-Wei Wu <yuwwu@indiana.edu>, and Yuzhen Ye <yye@indiana.edu>
=  Affiliation: School of Informatics, Indiana University, Bloomington
=
=  abundancebin is free software under the terms of the GNU General Public
=  License as published by the Free Software Foundation.
======================================*/

#include "profileManager.h"

int main(int argc, char *argv[])
{
	int i, composition_len, class_num, end_class_num, exclude, exclude_max;
	bool is_len, is_trial, is_reduce, is_recur, is_verbose, is_fasta;
	float threshold;
	char *inputfile, *outputfile;

	inputfile = NULL;
	composition_len = 20;
	outputfile = NULL;
	class_num = 0;
	end_class_num = 0;
	exclude = 0;
	exclude_max = 0;
	threshold = (float)0.7;
	is_len = true;
	is_trial = false;
	is_reduce = false;
	is_recur = false;
	is_verbose = false;
	is_fasta = false;
	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-input") == 0)
		{
			i++;
			inputfile = argv[i];
		}
		else if (strcmp(argv[i], "-kmer_len") == 0)
		{
			i++;
			composition_len = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-bin_num") == 0)
		{
			i++;
			class_num = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-exclude") == 0)
		{
			i++;
			exclude = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-exclude_max") == 0)
		{
			i++;
			exclude_max = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-trial") == 0)
		{
			is_trial = true;
			i++;
			class_num = atoi(argv[i]);
			i++;
			end_class_num = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-output") == 0)
		{
			i++;
			outputfile = argv[i];
		}
		else if (strcmp(argv[i], "-NO_LENGTH") == 0)
		{
			is_len = false;
		}
		else if (strcmp(argv[i], "-REDUCE_BINNUM") == 0)
		{
			is_reduce = true;
		}
		else if (strcmp(argv[i], "-RECURSIVE_CLASSIFICATION") == 0)
		{
			is_recur = true;
		}
		else if (strcmp(argv[i], "-VERBOSE") == 0)
		{
			is_verbose = true;
		}
		else if (strcmp(argv[i], "-OUTPUT_FASTA") == 0)
		{
			is_fasta = true;
		}
		else
		{
			//printf("Usage: EM_Coverage.exe -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-NO_LENGTH] [-REDUCE_CLASSNUM]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recurrent classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n\n\t\t(or undergo trial)\n\t\t-trial n1 n2 (guessed bin number range from n1 to n2, n1 < n2)");
#ifdef WIN32
			printf("Usage: AbundanceBin.exe -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#else
			printf("Usage: ./abundancebin -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#endif
			exit(-1);
		}
	}
	if (inputfile == NULL)
	{
		printf("Please supply input FASTA file.\n");
#ifdef WIN32
			printf("Usage: AbundanceBin.exe -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#else
			printf("Usage: ./abundancebin -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#endif
		exit(-1);
	}
	if (is_trial == false && is_recur == false && class_num == 0)
	{
		printf("Please supply the number of bins or undergo recursive classification.\n");
#ifdef WIN32
			printf("Usage: AbundanceBin.exe -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#else
			printf("Usage: ./abundancebin -input (input filename)\n\t\t[-kmer_len (composition len, default 20)]\n\t\t[-output (output file, default inputfile.log)]\n\t\t[-exclude (count)]\n\t\t[-exclude_max (count)]\n\t\t[-OUTPUT_FASTA]\n\n\t\t(if the bin number is known)\n\t\t-bin_num (bin number)\n\n\t\t(or undergo recursive classification)\n\t\t-RECURSIVE_CLASSIFICATION]\n");
#endif
		exit(-1);
	}

	profileManager pm(composition_len);
	if (outputfile != NULL)
	{
		pm.setOutputFile(outputfile);
	}
	pm.setIsLength(is_len);
	pm.setVerbose(is_verbose);
	pm.readFasta(inputfile);
	if (pm.isFasta() == false)
	{
		printf("Fasta file [%s] canot be opened. stop\n", inputfile);
		exit(-1);
	}

	if (is_recur == true)
	{
		i = pm.recursiveClassNum(exclude, exclude_max);
		// Doesn't need to call pm.classifySeq() and pm.writeKmerCount() because recursiveClassNum has already done that.

		if (i == 1)
		{
			printf("The sequences cannot be classified using recursive methodology. Put all sequences into one result file.\n");
		}
	}
	else if (is_trial == true)
	{
		pm.performTrial(class_num, end_class_num, exclude, exclude_max);
		pm.classifySeq(exclude, exclude_max, 0);
		pm.writeKmerCount();
	}
	else
	{
		if (is_reduce == true)
		{
			while (1)
			{
				pm.initializeEM(class_num);
				pm.performEM(exclude, exclude_max);
				i = pm.binNumberReduction(exclude, exclude_max);
				if (i == class_num)
				{
					break;
				}
				class_num = i;
			}
		}
		else
		{
			pm.initializeEM(class_num);
			pm.performEM(exclude, exclude_max);
			pm.classifySeq(exclude, exclude_max, 0);
		}
		pm.writeKmerCount();
		pm.writeBinFeature();
	}
	if (is_fasta == false)
	{
		pm.writeResult();
	}
	else
	{
		pm.writeFASTA();
	}

	return 0;
}
