/* =====================================
=  Program Name: abundancebin
=  File Name: myHash.cpp
=  Version: 1.0
=  Developer: Yu-Wei Wu <yuwwu@indiana.edu>, and Yuzhen Ye <yye@indiana.edu>
=  Affiliation: School of Informatics, Indiana University, Bloomington
=
=  abundancebin is free software under the terms of the GNU General Public
=  License as published by the Free Software Foundation.
======================================*/

#include "myHash.h"
#include <math.h>

myHash::myHash(int unitSize)
{
	mem_handler = new myMalloc();

	if (unitSize % 2 == 0)
	{
		hashSize = (int)pow(2, unitSize);
		//hashSize = (int)pow(2, unitSize + 4);
	}
	else
	{
		hashSize = (int)pow(2, unitSize + 1);
		//hashSize = (int)pow(2, unitSize + 5);
	}
	profileHead = (profile_lnk**)mem_handler->allocate(sizeof(profile_lnk*) * hashSize);
	memset(profileHead, '\0', sizeof(profile_lnk*) * hashSize);

	profileCurrent = (profile_lnk**)mem_handler->allocate(sizeof(profile_lnk*) * hashSize);
	memset(profileCurrent, '\0', sizeof(profile_lnk*) * hashSize);

	highest_count = 0;

	count = 0;
}

myHash::~myHash()
{
	int i;

	for (i = 0; i < hashSize; i++)
	{
		while (profileHead[i] != NULL)
		{
			profileCurrent[i] = profileHead[i]->next;
			free(profileHead[i]);
			profileHead[i] = profileCurrent[i];
		}
	}

	free(profileCurrent);
	free(profileHead);
	delete mem_handler;
}

void myHash::add(unsigned _int64 value)
{
	int i, j;

	i = value / hashSize;
	j = value % hashSize;

	if (highest_count == 0)
	{
		highest_count = 1;
	}
/*
	if (profileHead[i] == NULL)
	{
		profileHead[i] = (profile_lnk*)mem_handler->allocate(sizeof(profile_lnk));
		profileCurrent[i] = profileHead[i];
	}
	else
	{
		profileRecent = profileHead[i];
		while (profileRecent != NULL)
		{
			if (profileRecent->value == j)
			{
				profileRecent->count++;
				if (highest_count < profileRecent->count)
				{
					highest_count = profileRecent->count;
				}
				return;
			}
			profileRecent = profileRecent->next;
		}
		profileCurrent[i]->next = (profile_lnk*)mem_handler->allocate(sizeof(profile_lnk));
		profileCurrent[i] = profileCurrent[i]->next;
	}
	memset(profileCurrent[i], '\0', sizeof(profile_lnk));
	profileCurrent[i]->count = 1;
	profileCurrent[i]->value = j;
	count++;
*/

	if (profileHead[j] == NULL)
	{
		profileHead[j] = (profile_lnk*)mem_handler->allocate(sizeof(profile_lnk));
		profileCurrent[j] = profileHead[j];
	}
	else
	{
		profileRecent = profileHead[j];
		while (profileRecent != NULL)
		{
			if (profileRecent->value == i)
			{
				profileRecent->count++;
				if (highest_count < profileRecent->count)
				{
					highest_count = profileRecent->count;
				}
				return;
			}
			profileRecent = profileRecent->next;
		}
		profileCurrent[j]->next = (profile_lnk*)mem_handler->allocate(sizeof(profile_lnk));
		profileCurrent[j] = profileCurrent[j]->next;
	}
	memset(profileCurrent[j], '\0', sizeof(profile_lnk));
	profileCurrent[j]->count = 1;
	profileCurrent[j]->value = i;
	count++;

	return;
}

void myHash::resetIndex()
{
	currentHash = 0;
	profileRecent = NULL;
}

profile_lnk* myHash::next()
{
	if (profileRecent == NULL)
	{
		profileRecent = profileHead[currentHash];
		while (profileRecent == NULL && currentHash < hashSize)
		{
			currentHash++;
			profileRecent = profileHead[currentHash];
		}
		if (currentHash >= hashSize)
		{
			profileRecent = NULL;
		}
	}
	else
	{
		profileRecent = profileRecent->next;
		while (profileRecent == NULL && currentHash < hashSize)
		{
			currentHash++;
			profileRecent = profileHead[currentHash];
		}
		if (currentHash >= hashSize)
		{
			profileRecent = NULL;
		}
	}
	return profileRecent;
}

unsigned int myHash::nextCount()
{
	if (profileRecent == NULL)
	{
		profileRecent = profileHead[currentHash];
	}
	else
	{
		profileRecent = profileRecent->next;
		while (profileRecent == NULL && currentHash < hashSize)
		{
			currentHash++;
			profileRecent = profileHead[currentHash];
		}
	}
	if (profileRecent == NULL)
	{
		return 0;
	}
	else
	{
		return profileRecent->count;
	}
}

profile_lnk* myHash::find(unsigned _int64 value)
{
	int i, j;
/*
	i = value / hashSize;
	j = value % hashSize;

	profileRecent = NULL;
	if (profileHead[i] != NULL)
	{
		profileRecent = profileHead[i];
		while (profileRecent != NULL)
		{
			if (profileRecent->value == j)
			{
				break;
			}
			profileRecent = profileRecent->next;
		}
	}
*/
	i = value / hashSize;
	j = value % hashSize;

	profileRecent = NULL;
	if (profileHead[j] != NULL)
	{
		profileRecent = profileHead[j];
		while (profileRecent != NULL)
		{
			if (profileRecent->value == i)
			{
				break;
			}
			profileRecent = profileRecent->next;
		}
	}

	return profileRecent;
}

unsigned int myHash::reportTotalNum()
{
	return count;
}

unsigned _int64 myHash::reportTotalMem()
{
	return mem_handler->reportTotal();
}

unsigned int myHash::returnHighestCount()
{
	return highest_count;
}
