/* =====================================
=  Program Name: abundancebin
=  File Name: myStringHash.cpp
=  Version: 1.0
=  Developer: Yu-Wei Wu <yuwwu@indiana.edu>, and Yuzhen Ye <yye@indiana.edu>
=  Affiliation: School of Informatics, Indiana University, Bloomington
=
=  abundancebin is free software under the terms of the GNU General Public
=  License as published by the Free Software Foundation.
======================================*/

#include "myStringHash.h"

myStringHash::myStringHash()
{
	mem_handler = new myMalloc();

	hashSize = 256 * 256 * 100;
	hashHead = (strhash_lnk**)mem_handler->allocate(sizeof(strhash_lnk*) * hashSize);
	memset(hashHead, '\0', sizeof(strhash_lnk*) * hashSize);

	hashCurrent = (strhash_lnk**)mem_handler->allocate(sizeof(strhash_lnk*) * hashSize);
	memset(hashCurrent, '\0', sizeof(strhash_lnk*) * hashSize);
}

myStringHash::myStringHash(int unitSize)
{
	mem_handler = new myMalloc();

	hashSize = unitSize;
	hashHead = (strhash_lnk**)mem_handler->allocate(sizeof(strhash_lnk*) * hashSize);
	memset(hashHead, '\0', sizeof(strhash_lnk*) * hashSize);

	hashCurrent = (strhash_lnk**)mem_handler->allocate(sizeof(strhash_lnk*) * hashSize);
	memset(hashCurrent, '\0', sizeof(strhash_lnk*) * hashSize);
}

myStringHash::~myStringHash()
{
	int i;

	for (i = 0; i < hashSize; i++)
	{
		while (hashHead[i] != NULL)
		{
			hashCurrent[i] = hashHead[i]->next;
			free(hashHead[i]);
			hashHead[i] = hashCurrent[i];
		}
	}

	free(hashCurrent);
	free(hashHead);
	delete mem_handler;
}

void myStringHash::add(char *str, short destination)
{
	int i;

	i = getEncode(str);

	if (hashHead[i] == NULL)
	{
		hashHead[i] = (strhash_lnk*)mem_handler->allocate(sizeof(strhash_lnk));
		hashCurrent[i] = hashHead[i];
	}
	else
	{
		hashCurrent[i]->next = (strhash_lnk*)mem_handler->allocate(sizeof(strhash_lnk));
		hashCurrent[i] = hashCurrent[i]->next;
	}
	memset(hashCurrent[i], '\0', sizeof(strhash_lnk));
	hashCurrent[i]->ref = str;
	hashCurrent[i]->dest = destination;

	return;
}

short myStringHash::find(char *str)
{
	int i;

	i = getEncode(str);

	hashRecent = NULL;
	if (hashHead[i] != NULL)
	{
		hashRecent = hashHead[i];
		while (hashRecent != NULL)
		{
			if (strcmp(hashRecent->ref, str) == 0)
			{
				break;
			}
			hashRecent = hashRecent->next;
		}
	}
	return hashRecent->dest;
}

int myStringHash::getEncode(char *str)
{
	int i, len, encode;
	len = strlen(str);

	encode = 0;
	for (i = 0; i < len; i = i + 2)
	{
		if (i > len - 2) // only one character remains
		{
			encode = encode + ((int)(*(str + i)) * 256);
		}
		else
		{
			encode = encode + ((int)(*(str + i)) * 256) + (int)(*(str + i + 1));
		}
		if (len >= 100)
		{
			break;
		}
	}
	return encode;
}
