/* =====================================
=  Program Name: abundancebin
=  File Name: profileManager.h
=  Version: 1.0
=  Developer: Yu-Wei Wu <yuwwu@indiana.edu>, and Yuzhen Ye <yye@indiana.edu>
=  Affiliation: School of Informatics, Indiana University, Bloomington
=
=  abundancebin is free software under the terms of the GNU General Public
=  License as published by the Free Software Foundation.
======================================*/

#ifndef	__PROFILEMANAGER_H__
#define	__PROFILEMANAGER_H__

#include <string>
#include "myHash.h"
#include "myStringHash.h"
#include "myMalloc.h"
#include "myStack.h"
#include <time.h>
using namespace std;

typedef struct seq_lnk
{
	short class_destination;
	unsigned short recur_mark;
	char *header;
	struct seq_lnk *next;
} seq_lnk;

class profileManager
{
	public:
		// Functions
		profileManager(int input_len);
		~profileManager();
		void readFasta(char *inputFile);
		void setMerLen(int input_len);
		void setOutputFile(char *logfile);
		void initializeEM(int input_classnum);
		void performEM(int exclude, int exclude_max);
		void setIsLength(bool input);
		void setCoverageThreshold(float input);
		void setMaxRun(int maxRun);
		void setVerbose(bool verbose);
		bool isFasta();
		void classifySeq(int exclude, int exclude_max, int part);
		float getLikelihood();
		int performTrial(int start_class_num, int stop_class_num, int exclude, int exclude_max);
		void writeResult();
		void writeKmerCount();
		int binNumberReduction(int exclude, int exclude_max);
		int recursiveClassNum(int exclude, int exclude_max);
		void writeFASTA();
		void writeBinFeature();

	private:
		// Variables
		static const int PREDEFINED_MAX_GENOME_SIZE;
		static const int PREDEFINED_MIN_GENOME_SIZE;
		static const double CONST_E;
		static const float CONVERGE_THRESHOLD;
		static const float SMALL_VALUE;
		static const float VERY_SMALL_VALUE;
		static const int TYPE_INPUT;
		static const int TYPE_CLASSIFY;
		static const int TYPE_RECUR;
		static const int DEBUG_MUST;
		static const int DEBUG_NORMAL;
		myHash *profileHash;
		profile_lnk *profileRecent;
		seq_lnk *seqStart;
		seq_lnk *seqNow;
		char str[1024];
		char logstr[1024];
		char *inputFasta;
		char *logfile;
		unsigned int kmer_highest_count;
		int len_mer;
		int class_num;
		unsigned int seqnum;
		unsigned short EM_maxrun;
		bool is_len;
		bool is_Fasta;
		bool is_verbose;
		string *tempseq;
		fstream *log;
		short err;
		double **count_poisson_class;
		bool *usability;
		double *prevClassAbundance;
		double *classAbundance;
		double *prevClassSize;
		double *classSize;
		int *resultSeqNum;
		int *resultClass;
		time_t start_time;
		time_t end_time;
		float total_likelihood;
		float coverage_threshold;
		float kmer_threshold;
		float read_threshold;
		float abundance_diff_threshold;
		float size_threshold;
		float min_seq_threshold;
		int trial_class_num;
		myMalloc *mem_handler;
		// Functions
		bool parseFasta(char *inputFile, int type, int exclude, int exclude_max, int part);
		void initParam();
		void delPoissonClass();
		unsigned _int64 calNum(char *str);
		void writelog(char *ch, int debug_type);
		void writelog(float f, int debug_type);
		void writelog(double f, int debug_type);
		void writelog(int i, int debug_type);
		void writelog(unsigned _int64 i, int debug_type);
		void resetRecurMark();
		void revSeq(char *seq, char *result);
		void logElapsedTime();
};

#endif
